@@ -1,4 +1,4 @@
-from typing import Tuple, Dict, Union
+from typing import Tuple, Dict, Union, List
 import seaborn as sns
 import matplotlib
 
@@ -13,16 +13,24 @@ alpha_sns: float = 0.4
 d_unit: str = "mm"
 
 # Matplotlib default font
-fontsize: int = 16
-labelsize: int = 16
-titlesize: int = 20
+# fontsize: int = 18
+# labelsize: int = 19
+# titlesize: int = 20
+
+fontsize: int = 12
+labelsize: int = 12
+titlesize: int = 22
+
 
 fontweigh: str = "normal"
-font = {"weight": "normal", "size": fontsize, "family": "sans-serif"}
+font: Dict[str, Union[str, int]] = {"weight": "normal", "size": fontsize, "family": "sans-serif"}
 
 # Matplotlib colormaps
 cmap: str = "jet"
 
+# Plot colors
+colors: List[str] = list(matplotlib.colors.TABLEAU_COLORS.values())  # type: ignore
+
 # Histogram XY ratio
 xy_golden_ratio: float = 1.5
 hist_scale = 6
