@@ -1,26 +1,15 @@
 import matplotlib.axis
 import torch
 from torch import Tensor
-from typing import Tuple, Optional, Union
+from typing import Tuple, Optional, Union, Dict, List
 import matplotlib
 import numpy as np
 import matplotlib.pyplot as plt
 from mpl_toolkits.axes_grid1 import make_axes_locatable
-import seaborn as sns
+from muograph.plotting.style import set_plot_style, add_colorbar_right, colors
 
 from muograph.volume.volume import Volume
-from muograph.plotting.params import (
-    font,
-    d_unit,
-    scale,
-    cmap,
-    fontsize,
-    hist_figsize,
-    n_bins,
-    labelsize,
-    titlesize,
-    configure_plot_theme,
-)
+from muograph.plotting.params import cmap, scale, n_bins, d_unit
 
 
 class VoxelPlotting:
@@ -194,63 +183,53 @@ class VoxelPlotting:
         voi: Volume,
         xyz_voxel_preds: Tensor,
         xyz_voxel_pred_uncs: Optional[Tensor] = None,
-        voi_slice: Union[int, Tuple[int, int]] = 0,
+        voi_slice: Optional[Tuple[int, int]] = None,
         dim: int = 2,
         figname: Optional[str] = None,
         reverse: bool = False,
-        pred_label: str = "Scattering density",
-        fig_suptitle: str = "Voxels predictions",
+        pred_label: str = "",
+        fig_suptitle: Optional[str] = None,
         pred_unit: str = "[a.u]",
-        scale: float = 7.0,
         cmap: str = cmap,
+        proj1d: bool = True,
         reference_value: Optional[float] = None,
         v_min_max: Optional[Tuple[float, float]] = None,
     ) -> None:
         r"""
-        Plot a 2D slice from 3D voxel-wise predictions with optional uncertainty estimates.
+        Plot a 2D slice from 3D voxel-wise predictions, with optional uncertainties and 1D projections.
 
-        This method generates a plot of voxel-wise predictions from a 3D volume along a specified
-        dimension (`dim`). It allows for optional visualization of uncertainty in the predictions
-        and provides the ability to average the predictions along x and y axes. Additional
-        customization options include setting colormaps, axis labels, and saving the plot to a file.
+        This method visualizes a 2D slice of a 3D tensor of voxel-wise predictions, taken along a specified
+        dimension (`dim`). It optionally supports displaying uncertainties, 1D projections along the orthogonal
+        axes, and a reference line. The figure can be customized in terms of colormap, title, units, and saved
+        as a file.
 
         Args:
-            - voi (`Volume`): An instance of the volume class.
-            - xyz_voxel_preds (`Tensor`): Tensor containing voxel predictions for the volume. (shape: [n_vox_x, n_vox_y, n_vox_z]).
-            - xyz_voxel_pred_uncs : (`Optional[Tensor]`): Optional 3D tensor of the same shape as `xyz_voxel_preds` containing prediction
-            uncertainties, by default None.
-            - voi_slice  (`Tuple[int, int]`):  Start and end indices for slicing along the specified dimension.
-                Defaults to None, which uses the full range of the dimension.
-            - dim (`int`): The dimension along which to slice the 3D voxel-wise predictions.
-            Must be 0, 1, or 2 corresponding to x, y, or z axes respectively.
-            - figname (`Optional[str]`): If provided, saves the plot to the given file name, by default None.
-            - reverse  (`bool`) Whether to reverse the colormap, by default False.
-            - pred_label (`Optional[str]`): Label for the prediction data, used in axis labels and title, by default 'Prediction'.
-            - pred_unit (`Optional[str]`): Unit of the predictions to be displayed in the colorbar and axis labels, by default '[unit]'.
-            - scale (`Optional[float]`): Scale factor for determining figure size, by default 1.0.
-            - cmap (`Optional[str]`): Colormap to use for the voxel prediction plot, by default 'jet'.
-            - reference_value (`Optional[float]`): Optional reference value to display as a horizontal/vertical line in the averaged
-            predictions plot, by default None.
-             - v_min_max ('Optional[Tuple]'): Sets the min and max values of the 2D histogram.
+            voi (`Volume`): An instance of the `Volume` class defining the geometry and voxel structure.
+            xyz_voxel_preds (`Tensor`): A 3D tensor of predicted values for each voxel, of shape (Vx, Vy, Vz).
+            xyz_voxel_pred_uncs (`Optional[Tensor]`): Optional 3D tensor of prediction uncertainties with the
+                same shape as `xyz_voxel_preds`. Default is `None`.
+            voi_slice (`Optional[Tuple[int, int]]`): Start and end indices for slicing along `dim`.
+                If `None`, the full range of the volume along that dimension is used.
+            dim (`int`): The dimension to slice along (0: x, 1: y, 2: z).
+            figname (`Optional[str]`): If provided, the plot is saved to this filename (with plane suffix and PDF format).
+            reverse (`bool`): Whether to reverse the colormap. Default is `False`.
+            pred_label (`str`): Label for the prediction (used in colorbar and projections). Default is an empty string.
+            fig_suptitle (`Optional[str]`): Title for the entire figure. If `None`, a default title is generated.
+            pred_unit (`str`): Unit of prediction values (e.g. "[a.u.]", "[GeV]"). Default is "[a.u]".
+            cmap (`str`): Matplotlib colormap to use. Default is `cmap` (must be defined earlier).
+            proj1d (`bool`): Whether to include 1D projections (top and right). Default is `True`.
+            reference_value (`Optional[float]`): If provided, a horizontal and vertical reference line is drawn in the projections.
+            v_min_max (`Optional[Tuple[float, float]]`): Optional fixed min/max values for the colormap scale.
         """
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
 
-        # Set default font
-        matplotlib.rc("font", **font)
+        set_plot_style()
 
         # Define colormap
         cmap = cmap + "_r" if reverse else cmap
 
+        if voi_slice is None:
+            voi_slice = (0, voi.n_vox_xyz[dim] - 1)
+
         # Get 2D slice from 3D xyz voxel-wise predictions
         xy_data = VoxelPlotting.get_2D_slice_from_3D(
             dim=dim,
@@ -332,13 +311,16 @@ class VoxelPlotting:
             },
         }
 
-        # Compute the figure size based on the plot xy ratio
-        figsize = VoxelPlotting.get_fig_size(voi=voi, nrows=1, ncols=1, dims=dim_mapping[dim]["xy_dims"], scale=scale)  # type: ignore
-
-        # Compute figure size based on the main axis size
-        figsize = VoxelPlotting.get_fig_size(voi=voi, nrows=1, ncols=1, dims=dim_mapping[dim]["xy_dims"], scale=scale)  # type: ignore
         # Create the main figure
-        fig, ax = plt.subplots(figsize=figsize)
+        fig, ax = plt.subplots()
+
+        # Set title
+        if fig_suptitle is None:
+            fig_suptitle = (
+                f"Voxel predictions\nfor volume slice {dim_mapping[dim]['dim_label']} "
+                + r"$\in$"
+                + f"[{voi_slice_coord[0]:.0f}, {voi_slice_coord[-1]:.0f}] {d_unit}"
+            )
 
         # Set 2D map contrast
         vmin = xy_data.ravel().min().detach().cpu().item() if v_min_max is None else v_min_max[0]
@@ -360,97 +342,126 @@ class VoxelPlotting:
         # Set axis labels
         ax.set_xlabel(f"Voxel ${dim_mapping[dim]['x_label']}$ location [mm]", fontweight="bold")
         ax.set_ylabel(f"Voxel ${dim_mapping[dim]['y_label']}$ location [mm]", fontweight="bold")
-        ax.tick_params(axis="both", labelsize=labelsize)
+        ax.tick_params(axis="both")
 
-        divider = make_axes_locatable(ax)
-        ax_histx = divider.append_axes("top", 1.0, pad=0.1, sharex=ax)
-        ax_histy = divider.append_axes("right", 1.0, pad=0.1, sharey=ax)
+        if proj1d:
+            divider = make_axes_locatable(ax)
+            ax_histx = divider.append_axes("top", 1.0, pad=0.1, sharex=ax)
+            ax_histy = divider.append_axes("right", 1.0, pad=0.1, sharey=ax)
 
-        # Remove axis labels for histograms
-        ax_histx.tick_params(axis="x", labelbottom=False, labelsize=labelsize)
-        ax_histy.tick_params(axis="y", labelleft=False, labelsize=labelsize)
+            # Create ax for colorbar
+            ax_cb = divider.append_axes("right", size="5%", pad=0.1)
+            _ = ax.figure.colorbar(im, cax=ax_cb, label=f"{pred_label} {pred_unit}")
 
-        # Set ticks position to top
-        ax_histy.xaxis.set_ticks_position("top")
-        ax_histy.xaxis.set_label_position("top")
-        # ax_histy.xaxis.tick_top()
+            # Remove axis labels for histograms
+            ax_histx.tick_params(axis="x", labelbottom=False)
+            ax_histy.tick_params(axis="y", labelleft=False)
 
-        # Set figure title
-        ax_histx.set_title(
-            f"{fig_suptitle}\nfor volume slice {dim_mapping[dim]['dim_label']} " + r"$\in$" + f"[{voi_slice_coord[0]:.0f}, {voi_slice_coord[-1]:.0f}] {d_unit}",
-            fontweight="bold",
-            fontsize=titlesize,
-            y=1.05,
-        )
+            # Set ticks position to top
+            ax_histy.xaxis.set_ticks_position("top")
+            ax_histy.xaxis.set_label_position("top")
 
-        # Plot the predictions averaged along the x and y axis
-        if xyz_voxel_pred_uncs is None:
-            ax_histx.scatter(
-                dim_mapping[dim]["x_vox_pos"],
-                dim_mapping[dim]["x_data"],
-                marker=".",
-            )
-            ax_histy.scatter(
-                dim_mapping[dim]["y_data"],
-                dim_mapping[dim]["y_vox_pos"],
-                marker=".",
+            # Set figure title
+            ax_histx.set_title(
+                fig_suptitle,
+                fontweight="bold",
+                y=1.05,
             )
 
+            # Plot the predictions averaged along the x and y axis
+            if xyz_voxel_pred_uncs is None:
+                ax_histx.errorbar(
+                    dim_mapping[dim]["x_vox_pos"],
+                    dim_mapping[dim]["x_data"],
+                    # xerr=voi.vox_width[dim_mapping[dim]["xy_dims"][0]].detach().cpu().numpy() / 2,
+                    fmt="+",
+                    markersize=4,
+                    # elinewidth=1,
+                    # capsize=2,
+                    color="black",
+                    label=r"Projection $z \rightarrow \mu$ count",
+                    linestyle="solid",
+                    alpha=0.4,
+                )
+                ax_histy.errorbar(
+                    dim_mapping[dim]["y_data"],
+                    dim_mapping[dim]["y_vox_pos"],
+                    # yerr=voi.vox_width[dim_mapping[dim]["xy_dims"][1]].detach().cpu().numpy() / 2,
+                    fmt="+",
+                    markersize=4,
+                    # elinewidth=1,
+                    # capsize=2,
+                    color="black",
+                    linestyle="solid",
+                    alpha=0.4,
+                )
+
+            else:
+                # Plot uncertainties if available
+                ax_histx.errorbar(
+                    dim_mapping[dim]["x_vox_pos"],
+                    dim_mapping[dim]["x_data"],
+                    xerr=voi.vox_width[dim_mapping[dim]["xy_dims"][0]].detach().cpu().numpy() / 2,  # type: ignore
+                    yerr=dim_mapping[dim]["x_data_uncs"],
+                    fmt="|",
+                    markersize=4,
+                    elinewidth=1,
+                    capsize=2,
+                    color="black",
+                    label=r"Projection $z \rightarrow \mu$ count",
+                )
+                ax_histy.errorbar(
+                    dim_mapping[dim]["y_data"],
+                    dim_mapping[dim]["y_vox_pos"],
+                    yerr=voi.vox_width[dim_mapping[dim]["xy_dims"][1]].detach().cpu().numpy() / 2,  # type: ignore
+                    xerr=dim_mapping[dim]["y_data_uncs"],
+                    fmt="|",
+                    markersize=4,
+                    elinewidth=1,
+                    capsize=2,
+                    color="black",
+                )
+
+            # Set same range for x and y histograms
+            if reference_value is not None:
+                min_pred_xy = min(
+                    np.min(dim_mapping[dim]["x_data"]),  # type: ignore
+                    np.min(dim_mapping[dim]["y_data"]),  # type: ignore
+                )
+                max_pred_xy = max(
+                    np.max(dim_mapping[dim]["x_data"]),  # type: ignore
+                    np.min(dim_mapping[dim]["y_data"]),  # type: ignore
+                    reference_value,
+                )
+                ax_histx.set_ylim(min_pred_xy * 0.90, max_pred_xy * 1.1)
+                ax_histy.set_xlim(min_pred_xy * 0.90, max_pred_xy * 1.1)
+
+                # Plot reference value
+                ax_histx.axhline(y=reference_value, color="red", alpha=0.5)
+                ax_histy.axvline(x=reference_value, color="red", alpha=0.5)
+
+            # Display grid
+            ax_histx.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
+            ax_histy.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
+
+            # Set axis labels
+            # ax_histx.set_ylabel(pred_label + " " + pred_unit, fontweight="bold")
+            # ax_histy.set_xlabel(pred_label + " " + pred_unit, fontweight="bold")
+
         else:
-            # Plot uncertainties if available
-            ax_histx.errorbar(
-                x=dim_mapping[dim]["x_vox_pos"],
-                y=dim_mapping[dim]["x_data"],
-                xerr=0,
-                yerr=dim_mapping[dim]["x_data_uncs"],
-                marker=".",
-                alpha=0.6,
-            )
-            ax_histy.errorbar(
-                x=dim_mapping[dim]["y_data"],
-                y=dim_mapping[dim]["y_vox_pos"],
-                xerr=dim_mapping[dim]["y_data_uncs"],
-                yerr=0,
-                marker=".",
-                alpha=0.6,
+            # Set figure title
+            fig.suptitle(
+                fig_suptitle,
+                fontweight="bold",
+                y=1.05,
             )
 
-        # Set same range for x and y histograms
-        if reference_value is not None:
-            min_pred_xy = min(
-                np.min(dim_mapping[dim]["x_data"]),  # type: ignore
-                np.min(dim_mapping[dim]["y_data"]),  # type: ignore
-            )
-            max_pred_xy = max(
-                np.max(dim_mapping[dim]["x_data"]),  # type: ignore
-                np.min(dim_mapping[dim]["y_data"]),  # type: ignore
-                reference_value,
-            )
-            ax_histx.set_ylim(min_pred_xy * 0.98, max_pred_xy * 1.02)
-            ax_histy.set_xlim(min_pred_xy * 0.98, max_pred_xy * 1.02)
-
-            # Plot reference value
-            ax_histx.axhline(y=reference_value, color="red", alpha=0.5)
-            ax_histy.axvline(x=reference_value, color="red", alpha=0.5)
-
-        ax_histx.set_ylim()
-        # Display grid
-        ax_histx.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
-        ax_histy.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
-
-        # Add colorbar
-        cbar_ax = fig.add_axes(rect=(0.95, 0.1, 0.03, 0.6))  # [left, bottom, width, height]
-        cbar = fig.colorbar(im, cax=cbar_ax)  # Attach colorbar to the custom axis
-        cbar.set_label(pred_label + " " + pred_unit, fontweight="bold")  # Colorbar label
-
-        # Set axis labels
-        ax_histx.set_ylabel(pred_label + " " + pred_unit, fontsize=fontsize, fontweight="bold")
-        ax_histy.set_xlabel(pred_label + " " + pred_unit, fontsize=fontsize, fontweight="bold")
+            add_colorbar_right(ax, im, label=pred_label + " " + pred_unit)
 
         # Save plot
         if figname is not None:
             plt.savefig(
-                figname + "_" + dim_mapping[dim]["plane"] + "_view",  # type: ignore
+                figname + "_" + dim_mapping[dim]["plane"] + "_view.pdf",  # type: ignore
                 bbox_inches="tight",
             )
         plt.show()
@@ -470,6 +481,7 @@ class VoxelPlotting:
         pred_unit: str = "",
         scale: float = scale,
         cmap: str = cmap,
+        extension: str = "pdf",
     ) -> None:
         """
         Plots voxel predictions by slicing through a specified dimension of the volume of interest.
@@ -495,30 +507,20 @@ class VoxelPlotting:
             - None: Displays the generated plots and optionally saves them to a file.
         """
 
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
+        if extension not in ["pdf", "png", "svg"]:
+            raise ValueError(f"Extension must be pdf, png or svg, got {extension}")
 
-        # Set default font
-        matplotlib.rc("font", **font)
+        set_plot_style()
 
         # Define colormap
         cmap = cmap + "_r" if reverse else cmap
 
         # The range of volume slice to plot
         if voi_slice is None:
-            voi_slice = (0, voi.n_vox_xyz[dim])
+            voi_slice = (0, voi.n_vox_xyz[dim] - 1)
 
         # The number of volume slice to take into account
-        nplots = int(voi_slice[1] - voi_slice[0])
+        nplots = int(voi_slice[1] - voi_slice[0]) + 1
 
         # If multiple slices per plot, adjust nplots accordingly
         if nslice_per_plot > 1:
@@ -574,16 +576,16 @@ class VoxelPlotting:
         # Mapping for dimension-specific attributes
         dim_mapping = {
             2: {
-                "slice_fn": lambda p, s: p[:, :, s : s + nslice_per_plot].sum(dim=2) / nslice_per_plot,
-                "z_min_fn": lambda v, s: v.voxel_edges[0, 0, s, 0, 2],
-                "z_max_fn": lambda v, s: v.voxel_edges[0, 0, s + nslice_per_plot - 1, 1, 2],
+                "slice_fn": lambda p, s: p[:, :, s : s + nslice_per_plot].sum(dim=2).detach().cpu().numpy() / nslice_per_plot,
+                "z_min_fn": lambda v, s: v.voxel_edges[0, 0, s, 0, 2].detach().cpu().numpy(),
+                "z_max_fn": lambda v, s: v.voxel_edges[0, 0, s + nslice_per_plot - 1, 1, 2].detach().cpu().numpy(),
                 "left_right_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 0],
-                    voi.voxel_edges[-1, 0, 0, 1, 0],
+                    voi.voxel_edges[0, 0, 0, 0, 0].detach().cpu().numpy(),
+                    voi.voxel_edges[-1, 0, 0, 1, 0].detach().cpu().numpy(),
                 ),
                 "bottom_top_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 1],
-                    voi.voxel_edges[0, -1, 0, 1, 1],
+                    voi.voxel_edges[0, 0, 0, 0, 1].detach().cpu().numpy(),
+                    voi.voxel_edges[0, -1, 0, 1, 1].detach().cpu().numpy(),
                 ),
                 "dim_label": "z",
                 "x_label": f"x [{d_unit}]",
@@ -591,16 +593,16 @@ class VoxelPlotting:
                 "plane": "XY",
             },
             1: {
-                "slice_fn": lambda p, s: p[:, s : s + nslice_per_plot].sum(dim=1) / nslice_per_plot,
-                "z_min_fn": lambda v, s: v.voxel_edges[0, s, 0, 0, 1],
-                "z_max_fn": lambda v, s: v.voxel_edges[0, s + nslice_per_plot - 1, 0, 1, 1],
+                "slice_fn": lambda p, s: p[:, s : s + nslice_per_plot].sum(dim=1).detach().cpu().numpy() / nslice_per_plot,
+                "z_min_fn": lambda v, s: v.voxel_edges[0, s, 0, 0, 1].detach().cpu().numpy(),
+                "z_max_fn": lambda v, s: v.voxel_edges[0, s + nslice_per_plot - 1, 0, 1, 1].detach().cpu().numpy(),
                 "left_right_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 0],
-                    voi.voxel_edges[-1, 0, 0, 1, 0],
+                    voi.voxel_edges[0, 0, 0, 0, 0].detach().cpu().numpy(),
+                    voi.voxel_edges[-1, 0, 0, 1, 0].detach().cpu().numpy(),
                 ),
                 "bottom_top_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 2],
-                    voi.voxel_edges[0, 0, -1, 1, 2],
+                    voi.voxel_edges[0, 0, 0, 0, 2].detach().cpu().numpy(),
+                    voi.voxel_edges[0, 0, -1, 1, 2].detach().cpu().numpy(),
                 ),
                 "dim_label": "y",
                 "x_label": f"x [{d_unit}]",
@@ -608,16 +610,16 @@ class VoxelPlotting:
                 "plane": "XZ",
             },
             0: {
-                "slice_fn": lambda p, s: p[s : s + nslice_per_plot].sum(dim=0) / nslice_per_plot,
-                "z_min_fn": lambda v, s: v.voxel_edges[s, 0, 0, 0, 0],
-                "z_max_fn": lambda v, s: v.voxel_edges[s + nslice_per_plot - 1, 0, 0, 1, 0],
+                "slice_fn": lambda p, s: p[s : s + nslice_per_plot].sum(dim=0).detach().cpu().numpy() / nslice_per_plot,
+                "z_min_fn": lambda v, s: v.voxel_edges[s, 0, 0, 0, 0].detach().cpu().numpy(),
+                "z_max_fn": lambda v, s: v.voxel_edges[s + nslice_per_plot - 1, 0, 0, 1, 0].detach().cpu().numpy(),
                 "left_right_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 1],
-                    voi.voxel_edges[0, -1, 0, 1, 1],
+                    voi.voxel_edges[0, 0, 0, 0, 1].detach().cpu().numpy(),
+                    voi.voxel_edges[0, -1, 0, 1, 1].detach().cpu().numpy(),
                 ),
                 "bottom_top_extent": (
-                    voi.voxel_edges[0, 0, 0, 0, 2],
-                    voi.voxel_edges[0, 0, -1, 1, 2],
+                    voi.voxel_edges[0, 0, 0, 0, 2].detach().cpu().numpy(),
+                    voi.voxel_edges[0, 0, -1, 1, 2].detach().cpu().numpy(),
                 ),
                 "dim_label": "x",
                 "x_label": f"y [{d_unit}]",
@@ -657,19 +659,18 @@ class VoxelPlotting:
 
             axs[i].set_title(
                 r"{} $\in$ [{:.0f},{:.0f}] {}".format(dim_label, z_min, z_max, d_unit),
-                fontsize=fontsize - 1,
             )
             axs[i].set_aspect("equal")
             axs[i].grid(False)
 
             # set axes x label
             if (i % ncols == 0) | (i == 0):
-                axs[i].set_ylabel(y_label, fontweight="bold")
+                axs[i].set_ylabel(y_label)
             # set axes y label
             if i >= nplots - ncols:
-                axs[i].set_xlabel(x_label, fontweight="bold")
+                axs[i].set_xlabel(x_label)
 
-            axs[i].tick_params(axis="both", labelsize=labelsize)
+            axs[i].tick_params(axis="both")
 
         # remove empty subplots
         for i in range(1, extra + 1):
@@ -679,24 +680,22 @@ class VoxelPlotting:
         # Add color bar
         cbar_ax = fig.add_axes(rect=(1.01, 0.15, 0.05, 0.7))
         cbar = fig.colorbar(im, cax=cbar_ax)
-        cbar.set_label(pred_label + " " + pred_unit, fontweight="bold")  # Colorbar label
+        cbar.set_label(pred_label + " " + pred_unit)  # Colorbar label
 
         plt.subplots_adjust(right=0.99)
 
         # Set title
         fig.suptitle(
-            fig_suptitle + f"\nvoxel size = {voi.vox_width} {d_unit}",
+            fig_suptitle + f"\nvoxel size = {tuple(voi.vox_width.detach().cpu().numpy())} {d_unit}",
             x=0.58,
             y=1,
-            fontweight="bold",
-            fontsize=titlesize,
             va="top",
         )
 
         # Save file
         if figname is not None:
             fig.savefig(
-                figname + "_" + dim_mapping[dim]["plane"] + "_view_slice",  # type: ignore
+                figname + "_" + dim_mapping[dim]["plane"] + f"_view_slice.{extension}",  # type: ignore
                 bbox_inches="tight",
             )
 
@@ -711,14 +710,11 @@ class VoxelPlotting:
         log: bool = False,
         n_bins: int = n_bins,
     ) -> None:
-        # Set default font
-        matplotlib.rc("font", **font)
-
         # Create figure
-        fig, ax = plt.subplots(figsize=hist_figsize)
+        fig, ax = plt.subplots()
 
         # Set figure title
-        fig.suptitle(title, fontweight="bold", fontsize=titlesize)
+        fig.suptitle(title)
 
         # Get data range
         range = (xyz_voxel_preds.min().item(), xyz_voxel_preds.max().item())
@@ -752,9 +748,9 @@ class VoxelPlotting:
         ax.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
 
         # Axis labels
-        ax.set_ylabel("Frequency", fontweight="bold")
-        ax.set_xlabel(x_label, fontweight="bold")
-        ax.tick_params(axis="both", labelsize=labelsize)
+        ax.set_ylabel("Frequency")
+        ax.set_xlabel(x_label)
+        ax.tick_params(axis="both")
 
         ax.legend()
         plt.tight_layout()
@@ -773,9 +769,7 @@ class VoxelPlotting:
         r"""
         Plot the voxel volume as a grid given the desired projection.
         """
-        # Configure plot theme
-        configure_plot_theme(font=font)  # type: ignore
-
+        set_plot_style()
         if ax is None:
             fig, ax = plt.subplots()
 
@@ -802,12 +796,12 @@ class VoxelPlotting:
                 "x_label": "x",
                 "y_label": "y",
                 "x_lim": (
-                    (voi.xyz_min[0] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[0] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[0] - 2 * voi.vox_width[0]).detach().cpu().item(),
+                    (voi.xyz_max[0] + 2 * voi.vox_width[0]).detach().cpu().item(),
                 ),
                 "y_lim": (
-                    (voi.xyz_min[1] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[1] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[1] - 2 * voi.vox_width[1]).detach().cpu().item(),
+                    (voi.xyz_max[1] + 2 * voi.vox_width[1]).detach().cpu().item(),
                 ),
                 "extent_x": (voi.xyz_min[0].detach().cpu().item(), voi.xyz_max[0].detach().cpu().item()),
                 "extent_y": (voi.xyz_min[1].detach().cpu().item(), voi.xyz_max[1].detach().cpu().item()),
@@ -824,12 +818,12 @@ class VoxelPlotting:
                 "x_label": "x",
                 "y_label": "z",
                 "x_lim": (
-                    (voi.xyz_min[0] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[0] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[0] - 2 * voi.vox_width[0]).detach().cpu().item(),
+                    (voi.xyz_max[0] + 2 * voi.vox_width[0]).detach().cpu().item(),
                 ),
                 "y_lim": (
-                    (voi.xyz_min[2] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[2] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[2] - 2 * voi.vox_width[2]).detach().cpu().item(),
+                    (voi.xyz_max[2] + 2 * voi.vox_width[2]).detach().cpu().item(),
                 ),
                 "extent_x": (voi.xyz_min[0].detach().cpu().item(), voi.xyz_max[0].detach().cpu().item()),
                 "extent_y": (voi.xyz_min[2].detach().cpu().item(), voi.xyz_max[2].detach().cpu().item()),
@@ -846,12 +840,12 @@ class VoxelPlotting:
                 "x_label": "y",
                 "y_label": "z",
                 "x_lim": (
-                    (voi.xyz_min[1] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[1] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[1] - 2 * voi.vox_width[1]).detach().cpu().item(),
+                    (voi.xyz_max[1] + 2 * voi.vox_width[1]).detach().cpu().item(),
                 ),
                 "y_lim": (
-                    (voi.xyz_min[2] - 2 * voi.vox_width).detach().cpu().item(),
-                    (voi.xyz_max[2] + 2 * voi.vox_width).detach().cpu().item(),
+                    (voi.xyz_min[2] - 2 * voi.vox_width[2]).detach().cpu().item(),
+                    (voi.xyz_max[2] + 2 * voi.vox_width[2]).detach().cpu().item(),
                 ),
                 "extent_x": (voi.xyz_min[1].detach().cpu().item(), voi.xyz_max[1].detach().cpu().item()),
                 "extent_y": (voi.xyz_min[2].detach().cpu().item(), voi.xyz_max[2].detach().cpu().item()),
@@ -894,3 +888,96 @@ class VoxelPlotting:
                 xmin=extent_x[0],
                 xmax=extent_x[1],
             )
+        # plt.show()
+
+    @staticmethod
+    def plot_3D_to_1D(
+        data_3D: Union[Tensor, List[Tensor]],
+        data_labels: Optional[List[str]] = None,
+        voi: Optional[Volume] = None,
+        dim: int = 2,
+        ylabel: str = "value",
+        figname: Optional[str] = None,
+        title: Optional[str] = None,
+        plot_mean: bool = False,
+    ) -> None:
+        """
+        Plots a 1D projection of 3D tensor data along a specified dimension.
+
+        Args:
+            data_3D (Tensor): The 3D tensor containing the data to plot.
+            voi (Optional[Volume]): Volume of interest, defining voxel centers. If None, a default is created.
+            dim (int): The dimension to project onto (0, 1, or 2).
+            ylabel (str): Label for the Y-axis.
+
+        Raises:
+            ValueError: If `dim` is not in {0, 1, 2}.
+        """
+
+        fig, ax = plt.subplots()
+
+        if voi is None:
+            nx, ny, nz = data_3D.size() if isinstance(data_3D, Tensor) else data_3D[0].size()
+            voi = Volume(position=(0.0, 0.0, 0.0), dimension=(nx, ny, nz), voxel_width=(1, 1, 1))
+
+        if dim not in {0, 1, 2}:
+            raise ValueError(f"Invalid dimension {dim}. Must be one of 0, 1, or 2.")
+
+        def plot_data(ax: matplotlib.axes._axes.Axes, voi: Volume, data: Tensor, color: str = "blue", data_label: Optional[str] = None) -> None:
+            dim_mapping: Dict[int, Dict[str, Union[str, np.ndarray]]] = {
+                0: {
+                    "x_pos": voi.voxel_centers[:, 0, 0, 0].detach().cpu().numpy(),
+                    "data": data.mean(dim=2).mean(dim=1).detach().cpu().numpy(),
+                },
+                1: {
+                    "x_pos": voi.voxel_centers[0, :, 0, 1].detach().cpu().numpy(),
+                    "data": data.mean(dim=2).mean(dim=0).detach().cpu().numpy(),
+                },
+                2: {
+                    "x_pos": voi.voxel_centers[0, 0, :, 2].detach().cpu().numpy(),
+                    "data": data.mean(dim=0).mean(dim=0).detach().cpu().numpy(),
+                },
+            }
+
+            mean = np.mean(dim_mapping[dim]["data"])  # type: ignore
+            label = data_label if data_label is not None else f"Mean = {mean:.3f}"
+
+            # Plot 1D data
+            ax.scatter(
+                dim_mapping[dim]["x_pos"],
+                dim_mapping[dim]["data"],
+                marker="+",
+                s=50,
+                label=label,
+                alpha=0.8,
+                color=color,
+            )
+
+        if isinstance(data_3D, list):
+            for i, data in enumerate(data_3D):
+                data_label = data_labels[i] if data_labels is not None else None
+                plot_data(ax=ax, voi=voi, data=data, color=colors[i], data_label=data_label)
+
+        elif isinstance(data_3D, Tensor):
+            plot_data(ax=ax, voi=voi, data=data_3D, color=colors[0])
+
+        xlabel: Dict[int, str] = {0: "x [" + d_unit + "]", 1: "y [" + d_unit + "]", 2: "z [" + d_unit + "]"}
+
+        # Axis label
+        ax.set_xlabel(xlabel[dim], fontweight="bold")  # type: ignore
+        ax.set_ylabel(ylabel, fontweight="bold")
+
+        # Title
+        if title is not None:
+            fig.suptitle(title, fontweight="bold")
+
+        # Legend
+        if plot_mean:
+            ax.legend(bbox_to_anchor=(1.0, 0.7))
+        else:
+            ax.legend()
+
+        # Save figure
+        if figname is not None:
+            plt.savefig(figname + "_" + xlabel[dim][0], bbox_inches="tight")
+        plt.show()
