@@ -1,20 +1,19 @@
-from typing import Optional, Tuple, List, Dict, Union
+from typing import Optional, Tuple, List
 import torch
 from torch import Tensor
 from copy import deepcopy
 from functools import partial
 import math
 from pathlib import Path
+from dataclasses import dataclass
 
 from muograph.utils.device import DEVICE
 from muograph.tracking.tracking import TrackingMST
-from muograph.reconstruction.poca import POCA
+from muograph.reconstruction.poca import POCA, POCAParams
 from muograph.volume.volume import Volume
 from muograph.reconstruction.voxel_inferer import AbsVoxelInferer
 
-value_type = Union[float, partial, Tuple[float, float], bool, int]
-bca_params_type = Dict[str, value_type]
-
+P_MEAN = 4.0  # GeV
 
 r"""
 Provides class for computing voxelized scattering density predictions
@@ -24,18 +23,18 @@ to detect high-Z material using cosmic muons, 2013 JINST 8 P10013,
 """
 
 
-class BCA(POCA, AbsVoxelInferer):
-    _hit_per_voxel: Optional[Tensor] = None  # (Nx, Ny, Nz)
+@dataclass
+class BCAParams(POCAParams):
+    n_max_per_vox: int = 50
+    n_min_per_vox: int = 3
+    score_method: partial = partial(torch.quantile, q=0.5)
+    metric_method: partial = partial(torch.log)
+    p_range: Tuple[float, float] = (0.0, 10000000)
+    dtheta_range: Tuple[float, float] = (0.0, math.pi / 3)
 
-    _bca_params: bca_params_type = {
-        "n_max_per_vox": 50,
-        "n_min_per_vox": 3,
-        "score_method": partial(torch.quantile, q=0.5),
-        "metric_method": partial(torch.log),  # type: ignore
-        "p_range": (0.0, 10000000),  # MeV
-        "dtheta_range": (0.0, math.pi / 3),
-        "use_p": False,
-    }
+
+class BCA(POCA[BCAParams], AbsVoxelInferer):
+    _params: BCAParams = BCAParams()
 
     _vars_to_save = ["xyz_voxel_pred", "n_poca_per_vox"]
 
@@ -44,6 +43,7 @@ class BCA(POCA, AbsVoxelInferer):
         voi: Volume,
         tracking: TrackingMST,
         output_dir: Optional[str] = None,
+        filename: Optional[str] = None,
     ) -> None:
         r"""
         Initializes the BCA object with an instance of the TrackingMST class.
@@ -56,7 +56,13 @@ class BCA(POCA, AbsVoxelInferer):
             and `hit_per_voxel` attributes in a hdf5 file.
         """
         AbsVoxelInferer.__init__(self, voi=voi, tracking=tracking)
-        POCA.__init__(self, tracking=tracking, voi=voi, output_dir=output_dir)
+        POCA.__init__(
+            self,
+            tracking=tracking,
+            voi=voi,
+            output_dir=output_dir,
+            filename=filename,
+        )
 
         self.bca_indices: Tensor = deepcopy(self.poca_indices)
         self.bca_poca_points: Tensor = deepcopy(self.poca_points)
@@ -80,7 +86,7 @@ class BCA(POCA, AbsVoxelInferer):
         return distances
 
     @staticmethod
-    def compute_scattering_momentum_weight(dtheta: Tensor, p: Optional[Tensor] = None) -> Tensor:
+    def compute_scattering_momentum_weight(dtheta: Tensor, p: Tensor) -> Tensor:
         r"""
         Compute weights based on the muon scattering angle and momentum (if availabe).
 
@@ -96,8 +102,14 @@ class BCA(POCA, AbsVoxelInferer):
         dtheta = dtheta.unsqueeze(1)
         p = p.unsqueeze(0)
 
+        # Normalize the momentum
+        if not (p == 1).all():
+            p_norm = p / P_MEAN
+        else:
+            p_norm = p
+
         if p is not None:
-            weights = dtheta * p * (dtheta * p).T
+            weights = dtheta * p_norm * (dtheta * p_norm).T
         else:
             weights = dtheta * dtheta.T
 
@@ -232,7 +244,8 @@ class BCA(POCA, AbsVoxelInferer):
         full_metric = torch.tril(
             torch.where(
                 (distance_metric != 0) & (scattering_weights != 0),
-                distance_metric / scattering_weights,
+                # distance_metric / scattering_weights,
+                scattering_weights / distance_metric,
                 0.0,
             )
         )
@@ -269,7 +282,7 @@ class BCA(POCA, AbsVoxelInferer):
              - score_list (List) List containing lists of scores for each voxel.
         """
 
-        # Initialize score tensor with zeros
+        # Initialize score tensor with large negative values
         score_list = torch.zeros(voi.n_vox_xyz, dtype=torch.int16, device=DEVICE).tolist()
 
         # Compute voxel indices and POCA point counts
@@ -278,7 +291,7 @@ class BCA(POCA, AbsVoxelInferer):
         unique_voxels, counts = torch.unique(flat_vox_indices, return_counts=True)
 
         # Filter voxels with at least n_min_per_vox points
-        valid_voxels = unique_voxels[counts > n_min_per_vox]
+        valid_voxels = unique_voxels[counts >= n_min_per_vox]
 
         # Convert flat indices back to 3D coordinates
         voxel_coords = torch.stack(
@@ -295,7 +308,7 @@ class BCA(POCA, AbsVoxelInferer):
             i, j, k = coord.tolist()
             score_list[i][j][k] = self.compute_vox_wise_metric(
                 metric_method=metric_method,
-                vox_id=torch.tensor([i, j, k]),
+                vox_id=torch.tensor([i, j, k], device=bca_indices.device),
                 use_p=use_p,
                 bca_indices=bca_indices,
                 poca_points=poca_points,
@@ -336,6 +349,10 @@ class BCA(POCA, AbsVoxelInferer):
                         # Count the number of POCA points (hits) in this voxel
                         hit_per_voxel[i, j, k] = len(voxel_scores)
 
+        # replace zero values by minimum value
+        min_value = torch.min(final_voxel_scores[final_voxel_scores != 0.0])
+        final_voxel_scores[final_voxel_scores == 0.0] = min_value
+
         return final_voxel_scores, hit_per_voxel
 
     def _filter_events(self, mask: Tensor) -> None:
@@ -355,7 +372,7 @@ class BCA(POCA, AbsVoxelInferer):
         Returns:
             Path: Path to the BCA directory.
         """
-        return Path(str(self.output_dir) + "/" + self.bca_name + "/")
+        return Path(str(self.output_dir) + "/" + self.name + "/")
 
     def get_xyz_voxel_pred(self) -> Tensor:
         """
@@ -366,7 +383,7 @@ class BCA(POCA, AbsVoxelInferer):
 
         Compute voxel-wise scattering density predictions.
 
-        Uses parameters stored in `_bca_params`. The algorithm calculates:
+        Uses parameters stored in `_params`. The algorithm calculates:
             - Scattering density predictions (`pred`).
             - Number of POCA points used for each voxel's prediction (`hit_per_voxel`).
 
@@ -385,7 +402,7 @@ class BCA(POCA, AbsVoxelInferer):
             self.nhit,
             self.nhit_rejected,
         ) = self.compute_low_theta_events_voxel_wise_mask(
-            n_max_per_voxel=int(self.bca_params["n_max_per_vox"]),  # type: ignore
+            n_max_per_voxel=int(self.params.n_max_per_vox),  # type: ignore
             voi=self.voi,
             bca_indices=self.bca_indices,
             dtheta=self.bca_tracks.dtheta,
@@ -393,35 +410,45 @@ class BCA(POCA, AbsVoxelInferer):
         self._filter_events(self.mask)
 
         # momentum cut
-        if self.bca_params["use_p"]:
-            p_mask = (self.bca_tracks.E > self.bca_params["p_range"][0]) & (  # type: ignore
-                self.bca_tracks.E < self.bca_params["p_range"][1]  # type: ignore
+        if self.params.use_p:
+            p_mask = (self.bca_tracks.E > self.params.p_range[0]) & (  # type: ignore
+                self.bca_tracks.E < self.params.p_range[1]  # type: ignore
             )
         else:
             p_mask = torch.ones_like(self.bca_tracks.dtheta, dtype=torch.bool, device=DEVICE)
 
         # scattering angle cut
-        dtheta_mask = (self.bca_tracks.dtheta > self.bca_params["dtheta_range"][0]) & (  # type: ignore
-            self.bca_tracks.dtheta < self.bca_params["dtheta_range"][1]  # type: ignore
+        dtheta_mask = (self.bca_tracks.dtheta > self.params.dtheta_range[0]) & (  # type: ignore
+            self.bca_tracks.dtheta < self.params.dtheta_range[1]  # type: ignore
         )
 
         # apply dtheta, p cuts
         self._filter_events(mask=p_mask & dtheta_mask)
 
+        # prepare variables
+        p_max = torch.quantile(self.bca_tracks.p, q=self.params.p_clamp)
+        p = torch.clamp(self.bca_tracks.p, max=p_max)
+
+        dtheta_max = torch.quantile(self.bca_tracks.dtheta, q=self.params.dtheta_clamp)
+        dtheta = torch.clamp(self.bca_tracks.dtheta, max=dtheta_max)
+
         # compute voxels distribution
         self.score_list = self.compute_voxels_distribution(
-            metric_method=self.bca_params["metric_method"],  # type: ignore
-            use_p=self.bca_params["use_p"],  # type: ignore
-            n_min_per_vox=self.bca_params["n_min_per_vox"],  # type: ignore
+            metric_method=self.params.metric_method,  # type: ignore
+            use_p=self.params.use_p,  # type: ignore
+            n_min_per_vox=self.params.n_min_per_vox,  # type: ignore
             voi=self.voi,
-            momentum=self.bca_tracks.E,
+            momentum=p,
             bca_indices=self.bca_indices,
             poca_points=self.bca_poca_points,
-            dtheta=self.bca_tracks.dtheta,
+            dtheta=dtheta,
         )
 
         # compute fina scores
-        pred, self._hit_per_voxel = self.compute_final_scores(score_list=self.score_list, score_method=self.bca_params["score_method"])  # type: ignore
+        pred, self._hit_per_voxel = self.compute_final_scores(score_list=self.score_list, score_method=self.params.score_method)  # type: ignore
+
+        pred_max = torch.quantile(pred, q=self.params.preds_clamp)
+        pred = torch.clamp(pred, max=pred_max)
 
         self._recompute_preds = False
 
@@ -433,53 +460,68 @@ class BCA(POCA, AbsVoxelInferer):
         r"""
         Returns the name of the bca given its parameters.
         """
+        name = self.get_string_params()
+        name = name.replace(", ", "_")
+
+        return name
+
+    def get_string_params(self) -> str:
+        """
+        Converts a dataclass-like params object into a readable string, including partial functions.
+        """
 
         def get_partial_name_args(func: partial) -> str:
-            r"""
+            """
             Returns the name, arguments and their value of a partial method as a string.
             """
             func_name = func.func.__name__
             args, values = list(func.keywords.keys()), list(func.keywords.values())
             for i, arg in enumerate(args):
-                func_name += "_{}={}".format(arg, values[i])
+                func_name += f"_{arg}={values[i]}"
             return func_name
 
-        method = "method_{}_".format(get_partial_name_args(self.bca_params["score_method"]))  # type: ignore
-        metric = "metric_{}_".format(get_partial_name_args(self.bca_params["metric_method"]))  # type: ignore
-        dtheta = "{:.1f}_{:.1f}_mrad_".format(self.bca_params["dtheta_range"][0] * 1000, self.bca_params["dtheta_range"][1] * 1000)  # type: ignore
-        dp = "{:.0f}_{:.0f}_MeV_".format(self.bca_params["p_range"][0], self.bca_params["p_range"][1])  # type: ignore
-        n_min_max = "n_min_max_{}_{}_".format(self.bca_params["n_min_per_vox"], self.bca_params["n_max_per_vox"])
-        use_p = "use_p_{}".format(self.bca_params["use_p"])
-
-        bca_name = method + metric + dtheta + dp + n_min_max + use_p
-        return bca_name
+        parts = []
+        for key, val in vars(self.params).items():
+            if isinstance(val, partial):
+                val_str = get_partial_name_args(val)
+            elif isinstance(val, (tuple, list)):
+                val_str = "_".join([str(v) for v in val])
+            else:
+                val_str = str(val)
+            parts.append(f"{key}={val_str}")
+        return ", ".join(parts)
 
     @property
-    def bca_name(self) -> str:
+    def name(self) -> str:
         r"""
         The name of the bca algorithm given its parameters
         """
         return self.get_bca_name()
 
     @property
-    def bca_params(self) -> Dict[str, value_type]:
+    def params(self) -> BCAParams:
         r"""
         The parameters of the bca algorithm.
         """
-        return self._bca_params
+        return self._params
 
-    @bca_params.setter
-    def bca_params(self, value: Dict[str, value_type]) -> None:
+    @params.setter
+    def params(self, value: BCAParams) -> None:
         r"""
         Sets the parameters of the bca algorithm.
         Args:
             - Dict containing the parameters name and value. Only parameters with
             valid name and non `None` values wil be updated.
         """
-        for key in value.keys():
-            if key in self._bca_params.keys():
-                if value[key] is not None:
-                    self._bca_params[key] = value[key]
+        if not isinstance(value, BCAParams):
+            raise TypeError("params must be an instance of BCAParams")
+
+        if not hasattr(self, "_params") or self._params is None:
+            self._params = BCAParams()
+
+        for key, val in value.__dict__.items():
+            if val is not None:
+                setattr(self._params, key, val)
 
         self._recompute_preds = True
 
