@@ -1,25 +1,25 @@
 import torch
 from torch import Tensor
 from copy import deepcopy
-from typing import Optional, Dict, Union
-import matplotlib
+from typing import Optional, Dict, Union, Tuple, Generic, TypeVar, cast
 import matplotlib.pyplot as plt
 from fastprogress import progress_bar
 import numpy as np
-import seaborn as sns
 import math
+from dataclasses import dataclass
 
 from muograph.utils.save import AbsSave
 from muograph.utils.device import DEVICE
 from muograph.utils.datatype import dtype_track, dtype_n
 from muograph.volume.volume import Volume
 from muograph.tracking.tracking import TrackingMST
-from muograph.plotting.voxel import VoxelPlotting
-from muograph.plotting.params import font
+from muograph.plotting.style import set_plot_style
+from muograph.utils.tools import normalize
+from muograph.reconstruction.voxel_inferer import AbsVoxelInferer
 
 
 r"""
-Provides class for copmuting POCA locations and POCA-based voxelized scattering density predictions
+Provides class for computing POCA locations and POCA-based voxelized scattering density predictions
 """
 
 
@@ -28,7 +28,18 @@ def are_parallel(v1: Tensor, v2: Tensor, tol: float = 1e-5) -> bool:
     return bool(torch.all(torch.abs(cross_prod) < tol).detach().cpu().item())
 
 
-class POCA(AbsSave, VoxelPlotting):
+@dataclass
+class POCAParams:
+    use_p: bool = False
+    p_clamp: float = 0.999
+    dtheta_clamp: float = 0.999
+    preds_clamp: float = 0.999
+
+
+P = TypeVar("P", bound="POCAParams")
+
+
+class POCA(AbsSave, AbsVoxelInferer, Generic[P]):
     r"""
     A class for Point Of Closest Approach computation in the context of a Muon Scattering Tomography analysis.
     """
@@ -38,8 +49,9 @@ class POCA(AbsSave, VoxelPlotting):
     _n_poca_per_vox: Optional[Tensor] = None  # (nx, ny, nz)
     _poca_indices: Optional[Tensor] = None  # (mu, 3)
     _mask_in_voi: Optional[Tensor] = None  # (mu)
+    _poca_xyz_voxel_preds: Optional[Tensor] = None  # (nvox_x, nvox_y, nvox_z)
 
-    _batch_size: int = 1024
+    _batch_size: int = 2048
 
     _vars_to_save = [
         "poca_points",
@@ -53,51 +65,45 @@ class POCA(AbsSave, VoxelPlotting):
         "poca_indices",
     ]
 
+    # _params: POCAParams = POCAParams()
+    _params: Optional[P] = None
+
+    _recompute_preds: bool = True
+
     def __init__(
         self,
-        tracking: Optional[TrackingMST] = None,
-        voi: Optional[Volume] = None,
-        poca_file: Optional[str] = None,
+        voi: Volume,
+        tracking: TrackingMST,
         output_dir: Optional[str] = None,
+        filename: Optional[str] = None,
     ) -> None:
-        r"""
-        Initializes the POCA object with either an instance of the TrackingMST class or a
-        poca.hdf5 file.
+        """
+        Initializes the POCA object with either a TrackingMST instance.
 
         Args:
-            - tracking (Optional[TrackingMST]): Instance of the TrackingMST class.
-            - voi (Optional[Volume]): Instance of the Volume class. If provided, muon events with
-            poca locations outside the voi will be filtered out, the number of poca locations per voxel
-            `n_poca_per_vox` as well as the voxel indices of each poca location will be computed.
-            poca_file (Optional[str]): The path to the poca.hdf5 to load attributes from.
-            - output_dir (Optional[str]): Path to a directory where to save POCA attributes
-            in a hdf5 file.
+            tracking (Optional[TrackingMST]): Muon tracking data.
+            voi (Optional[Volume]): Volume of interest. If provided, POCA points outside the VOI will be filtered.
+            output_dir (Optional[str]): Directory to save POCA attributes.
+            filename (Optional[str]): Filename for saving if output_dir is provided.
         """
         AbsSave.__init__(self, output_dir=output_dir)
-        VoxelPlotting.__init__(self, voi)
+        AbsVoxelInferer.__init__(self, voi=voi, tracking=tracking)
 
-        if tracking is None and poca_file is None:
-            raise ValueError("Provide either poca.hdf5 file of TrackingMST instance.")
+        self.filename = filename if filename else "poca"
 
-        # Compute poca attributes if TrackingMST is provided
-        elif (tracking is not None) and (voi is not None):
-            self.tracks = deepcopy(tracking)
-            self.voi = voi
+        self.tracks = deepcopy(tracking)
+        self.voi = voi
 
-            # Remove parallel events
-            self.tracks._filter_muons(self.parallel_mask)
+        # Remove parallel events
+        self.tracks._filter_muons(self.parallel_mask)
 
-            # Remove POCAs outside voi
-            self.tracks._filter_muons(self.mask_in_voi)
-            self._filter_pocas(self.mask_in_voi)
+        # Remove POCAs outside voi
+        self.tracks._filter_muons(self.mask_in_voi)
+        self._filter_pocas(self.mask_in_voi)
 
-            # Save attributes to hdf5
-            if output_dir is not None:
-                self.save_attr(self._vars_to_save, self.output_dir, filename="poca")
-
-        # Load poca attributes from hdf5 if poca_file is provided
-        elif tracking is None and poca_file is not None:
-            self.load_attr(self._vars_to_load, poca_file)
+        # Save attributes to hdf5
+        if output_dir is not None:
+            self.save_attr(self._vars_to_save, self.output_dir, filename=self.filename)
 
     def __repr__(self) -> str:
         return f"Collection of {self.n_mu} POCA locations."
@@ -113,7 +119,7 @@ class POCA(AbsSave, VoxelPlotting):
             tol: Tolerance value for determining if tracks are parallel.
 
         Returns:
-            mask: Boolean tensor with size (n_mu), where True indicates the event is not parallel.
+        Tensor: Boolean mask of shape (n_mu,), True where tracks are **not parallel**.
         """
         # Compute the cross product for all pairs at once
         cross_prod = torch.linalg.cross(tracks_in, tracks_out, dim=1)
@@ -125,13 +131,16 @@ class POCA(AbsSave, VoxelPlotting):
 
     def _filter_pocas(self, mask: Tensor) -> None:
         r"""
-        Remove poca points specified as False in `mask`.
+        Removes POCA points where `mask` is False.
 
-        Arguments:
-            mask: (N,) Boolean tensor. poca points with False elements will be removed.
+        Args:
+            mask (Tensor): Boolean tensor of shape (n_mu). Only POCA points where mask is True are kept.
         """
         self.poca_points = self.poca_points[mask]
 
+        if self._poca_indices is not None:
+            self.poca_indices = self.poca_indices[mask]
+
     @staticmethod
     def compute_poca_points(points_in: Tensor, points_out: Tensor, tracks_in: Tensor, tracks_out: Tensor) -> Tensor:
         """
@@ -205,14 +214,15 @@ class POCA(AbsSave, VoxelPlotting):
     @staticmethod
     def assign_voxel_to_pocas(poca_points: Tensor, voi: Volume, batch_size: int) -> Tensor:
         """
-        Get the indinces of the voxel corresponding to each poca point.
+        Assign voxel indices to each POCA point.
 
-        Arguments:
-            - poca_points: Tensor with size (n_mu, 3).
-            - voi: An instance of the VolumeInterest class.
+        Args:
+            poca_points (Tensor): POCA locations, shape (n_mu, 3).
+            voi (Volume): Volume of interest.
+            batch_size (int): Number of POCA points to process per batch.
 
         Returns:
-            - poca points voxel indices as List[List[int]] with length n_mu.
+            Tensor: Voxel indices for each POCA point, shape (n_mu, 3), with -1 for out-of-bounds points.
         """
         indices = torch.ones((len(poca_points), 3), dtype=dtype_n, device=poca_points.device) * -1
 
@@ -251,77 +261,71 @@ class POCA(AbsSave, VoxelPlotting):
         return indices
 
     @staticmethod
-    def compute_n_poca_per_vox(poca_points: Tensor, voi: Volume) -> Tensor:
-        """
-        Computes the number of POCA points per voxel, given a voxelized volume VOI.
+    def compute_n_poca_per_vox(poca_indices: Tensor, voi: Volume) -> Tensor:
+        nx, ny, nz = voi.n_vox_xyz
 
-        Arguments:
-         - voi:VolumeIntrest, an instance of the VOI class.
-         - poca_points: Tensor containing the poca points location, with size (n_mu, 3).
+        flat_indices = poca_indices[:, 0] * ny * nz + poca_indices[:, 1] * nz + poca_indices[:, 2]
 
-        Returns:
-         - n_poca_per_vox: torch.tensor(dtype=int64) with size (nvox_x,nvox_y,nvox_z),
-         the number of poca points per voxel.
-        """
-
-        n_poca_per_vox = torch.zeros(tuple(voi.n_vox_xyz), device=DEVICE, dtype=dtype_n)
-
-        for i in range(voi.n_vox_xyz[2]):
-            z_min = voi.xyz_min[2] + i * voi.vox_width
-            z_max = z_min + voi.vox_width
-            mask_slice = (poca_points[:, 2] >= z_min) & ((poca_points[:, 2] <= z_max))
-
-            H, _, __ = np.histogram2d(
-                poca_points[mask_slice, 0].detach().cpu().numpy(),
-                poca_points[mask_slice, 1].detach().cpu().numpy(),
-                bins=(int(voi.n_vox_xyz[0]), int(voi.n_vox_xyz[1])),
-                range=(
-                    (
-                        voi.xyz_min[0].detach().cpu().numpy(),
-                        voi.xyz_max[0].detach().cpu().numpy(),
-                    ),
-                    (
-                        voi.xyz_min[1].detach().cpu().numpy(),
-                        voi.xyz_max[1].detach().cpu().numpy(),
-                    ),
-                ),
-            )
-
-            n_poca_per_vox[:, :, i] = torch.tensor(H, dtype=dtype_n)
-
-        return n_poca_per_vox
+        return torch.bincount(flat_indices, minlength=nx * ny * nz).reshape(voi.n_vox_xyz)
 
     @staticmethod
     def compute_mask_in_voi(poca_points: Tensor, voi: Volume) -> Tensor:
         """
-        Compute the mask of POCA points located within the volumne of interest.
+        Compute a boolean mask indicating which POCA points lie inside the VOI.
 
-        Arguments:
-            - poca_points: Tensor with size (n_mu, 3).
-            - voi: An instance of the VolumeInterest class.
+        Args:
+            poca_points (Tensor): POCA locations, shape (n_mu, 3).
+            voi (Volume): Volume of interest.
 
         Returns:
-            - mask_in_voi: Tensor of bool with size (n_mu) with True if
-            the POCA point is within the voi, False otherwise.
+            Tensor: Boolean mask, shape (n_mu,).
         """
 
         masks_xyz = [(poca_points[:, i] >= voi.xyz_min[i]) & (poca_points[:, i] <= voi.xyz_max[i]) for i in range(3)]
         return masks_xyz[0] & masks_xyz[1] & masks_xyz[2]
 
-    def plot_poca_event(self, event: int, proj: str = "XZ", voi: Optional[Volume] = None, figname: Optional[str] = None) -> None:
-        matplotlib.rc("font", **font)
+    @staticmethod
+    def compute_full_mask(mask_in_voi: Tensor, parallel_mask: Tensor) -> Tensor:
+        """
+        Combine `mask_in_voi` and `parallel_mask` to get a final valid-event mask.
 
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
+        Args:
+            mask_in_voi (Tensor): POCA-in-VOI mask.
+            parallel_mask (Tensor): Non-parallel-track mask.
+
+        Returns:
+            Tensor: Combined boolean mask.
+        """
+
+        full_mask = torch.zeros_like(parallel_mask, dtype=torch.bool, device=DEVICE)
+        full_mask[torch.where(parallel_mask)[0][mask_in_voi]] = True
+
+        return full_mask
+
+    def get_name_from_params(self) -> str:
+        """
+        Generate a string name for the POCA instance based on its parameters.
+
+        Args:
+            asr_params (POCAParams): POCA parameters.
+        """
+        method = "POCA_"
+        name = self.get_string_params()
+        name = name.replace(", ", "_")
+        return method + name
+
+    def plot_poca_event(self, event: int, proj: str = "XZ", voi: Optional[Volume] = None, figname: Optional[str] = None) -> None:
+        """
+        Plot a single muon event and its POCA location in 2D (XZ or YZ projection).
+
+        Args:
+            event (int): Index of the event to plot.
+            proj (str): "XZ" or "YZ". Defaults to "XZ".
+            voi (Optional[Volume]): If provided, VOI will be plotted.
+            figname (Optional[str]): If provided, saves the figure to this path.
+        """
+
+        set_plot_style()
 
         dim_map: Dict[str, Dict[str, Union[str, int]]] = {
             "XZ": {"x": 0, "y": 2, "xlabel": r"$x$ [mm]", "ylabel": r"$z$ [mm]", "proj": "XZ"},
@@ -330,14 +334,13 @@ class POCA(AbsSave, VoxelPlotting):
 
         dim_xy = (int(dim_map[proj]["x"]), int(dim_map[proj]["y"]))
 
-        fig, ax = plt.subplots(figsize=(10, 5))
+        fig, ax = plt.subplots()
         fig.suptitle(
             f"Tracking of event {event:,d}"
             + "\n"
             + f"{dim_map[proj]['proj']} projection, "
             + r"$\delta\theta$ = "
             + f"{self.tracks.dtheta[event] * 180 / math.pi:.2f} deg",
-            fontweight="bold",
         )
 
         points_in_np = self.tracks.points_in.detach().cpu().numpy()
@@ -384,13 +387,125 @@ class POCA(AbsSave, VoxelPlotting):
             bbox_to_anchor=(1.0, 0.7),
         )
         ax.set_aspect("equal")
-        ax.set_xlabel(f"{dim_map[proj]['xlabel']}", fontweight="bold")
-        ax.set_ylabel(f"{dim_map[proj]['ylabel']}", fontweight="bold")
+        ax.set_xlabel(f"{dim_map[proj]['xlabel']}")
+        ax.set_ylabel(f"{dim_map[proj]['ylabel']}")
 
         if figname is not None:
             plt.savefig(figname, bbox_inches="tight")
         plt.show()
 
+    def get_xyz_voxel_pred(self) -> Tensor:
+        r"""
+        Compute the per-voxel prediction of scattering strength as the root mean square (RMS)
+        of scattering angles, with optional momentum weighting.
+
+        For each track, the scattering angle (``dtheta``) and, if enabled, the momentum (``p``)
+        are clamped at quantile thresholds defined in ``self.params``. These values are
+        then combined into a per-track score, which is accumulated voxel-wise according to
+        the track's POCA-assigned voxel index. The RMS of the scores is computed per voxel.
+
+        Steps:
+            1. Clamp ``dtheta`` values at the quantile threshold ``dtheta_clamp``.
+            2. If ``use_p`` is True:
+                - Clamp ``p`` values at ``p_clamp``.
+                - Compute the per-track score as:
+                ``score = (dtheta ** 2) * (p ** 2) / (p.mean() ** 2)``
+            3. If ``use_p`` is False:
+                - The score is simply ``dtheta ** 2``.
+            4. For each voxel, aggregate track scores assigned to that voxel, and compute the
+            RMS value: ``sqrt(sum(score) / count)``.
+            5. Clamp the final voxel RMS values at the quantile threshold ``preds_clamp``.
+
+        Returns:
+            Tensor: A 3D tensor of shape ``(nx, ny, nz)``, where each element is the clamped RMS
+            scattering score of tracks assigned to the corresponding voxel. Voxels without
+            assigned tracks are set to 0.
+        """
+
+        dtheta_max = torch.quantile(self.tracks.dtheta, q=self.params.dtheta_clamp)
+        dtheta = torch.clamp(self.tracks.dtheta, max=dtheta_max)
+
+        if self.params.use_p:
+            p_max = torch.quantile(self.tracks.p, q=self.params.p_clamp)
+            p = torch.clamp(self.tracks.p, max=p_max)
+            dtheta = torch.clamp(self.tracks.dtheta, max=dtheta_max)
+            # score = (dtheta ** 2) * (torch.log(p ** 2)) / torch.log(p.mean() ** 2)
+            score = (dtheta**2) * ((p**2)) / (p.mean() ** 2)
+
+        else:
+            score = dtheta**2
+
+        nx, ny, nz = self.voi.n_vox_xyz
+
+        flat_indices = self.poca_indices[:, 0] * ny * nz + self.poca_indices[:, 1] * nz + self.poca_indices[:, 2]
+
+        # Count number of entries per voxel
+        counts = torch.bincount(flat_indices, minlength=nx * ny * nz)
+
+        sums = torch.bincount(flat_indices, weights=score, minlength=nx * ny * nz)
+
+        # Avoid division by zero
+        mask = counts > 0
+        rms_values = torch.zeros_like(sums)
+        rms_values[mask] = torch.sqrt(sums[mask] / counts[mask])
+
+        voxel_rms = rms_values.reshape(nx, ny, nz)
+
+        self._recompute_preds = False
+
+        voxel_rms_max = torch.quantile(voxel_rms, q=self.params.preds_clamp)
+        voxel_rms = torch.clamp(voxel_rms, max=voxel_rms_max)
+
+        return voxel_rms
+
+    def get_tracks_in_voxel(self, voxel_idx: Tuple[int, int, int]) -> Tuple[Tensor, Tensor, Optional[Tensor]]:
+        """
+        Given a voxel index (i, j, k), return:
+        - track indices contributing to that voxel
+        - corresponding dtheta values
+        - corresponding p values (if used)
+        """
+        i, j, k = voxel_idx
+        nx, ny, nz = self.voi.n_vox_xyz
+
+        # Compute flat voxel index
+        voxel_flat = i * ny * nz + j * nz + k
+
+        flat_indices = self.poca_indices[:, 0] * ny * nz + self.poca_indices[:, 1] * nz + self.poca_indices[:, 2]
+
+        # Mask for tracks in that voxel
+        mask = flat_indices == voxel_flat
+
+        # Get indices of contributing tracks
+        track_indices = torch.nonzero(mask, as_tuple=True)[0]
+
+        # dtheta values (clamped as in voxel_rms computation)
+        dtheta_max = torch.quantile(self.tracks.dtheta, q=self.params.dtheta_clamp)
+        dtheta = torch.clamp(self.tracks.dtheta, max=dtheta_max)[mask]
+
+        # p values (clamped if enabled)
+        if self.params.use_p:
+            p_max = torch.quantile(self.tracks.p, q=self.params.p_clamp)
+            p = torch.clamp(self.tracks.p, max=p_max)[mask]
+        else:
+            p = None
+
+        return track_indices, dtheta, p
+
+    def get_string_params(self) -> str:
+        """
+        Converts a dataclass-like params object into a readable string, including partial functions.
+        """
+
+        parts = []
+        for key, val in vars(self.params).items():
+            if isinstance(val, (tuple, list)):
+                val_str = "_".join([str(v) for v in val])
+            else:
+                val_str = str(val)
+            parts.append(f"{key}={val_str}")
+        return ", ".join(parts)
+
     @property
     def n_mu(self) -> int:
         r"""The number of muons."""
@@ -398,6 +513,7 @@ class POCA(AbsSave, VoxelPlotting):
 
     @property
     def poca_points(self) -> Tensor:
+        r"""Tensor: The POCA points computed from the incoming and outgoing tracks."""
         if self._poca_points is None:
             self._poca_points = self.compute_poca_points(
                 points_in=self.tracks.points_in,
@@ -409,36 +525,87 @@ class POCA(AbsSave, VoxelPlotting):
 
     @poca_points.setter
     def poca_points(self, value: Tensor) -> None:
+        r"""Set the POCA points."""
         self._poca_points = value
 
     @property
     def mask_in_voi(self) -> Tensor:
+        r"""Tensor: The mask indicating which POCA points are within the volume of interest (VOI)."""
         if self._mask_in_voi is None:
             self._mask_in_voi = self.compute_mask_in_voi(poca_points=self.poca_points, voi=self.voi)
         return self._mask_in_voi
 
     @property
     def n_poca_per_vox(self) -> Tensor:
+        r"""Tensor: The number of POCA points per voxel."""
         if self._n_poca_per_vox is None:
-            self._n_poca_per_vox = self.compute_n_poca_per_vox(poca_points=self.poca_points, voi=self.voi)
+            self._n_poca_per_vox = self.compute_n_poca_per_vox(poca_indices=self.poca_indices, voi=self.voi)
         return self._n_poca_per_vox
 
     @n_poca_per_vox.setter
     def n_poca_per_vox(self, value: Tensor) -> None:
+        r"""Set the number of POCA points per voxel."""
         self._n_poca_per_vox = value
 
+    @property
+    def n_poca_per_vox_norm(self) -> Tensor:
+        """
+        Normalized count of POCA points per voxel.
+        """
+        return normalize(self.n_poca_per_vox)  # type: ignore
+
     @property
     def poca_indices(self) -> Tensor:
+        r"""Tensor: The indices of the POCA points assigned to each voxel."""
         if self._poca_indices is None:
             self._poca_indices = self.assign_voxel_to_pocas(poca_points=self.poca_points, voi=self.voi, batch_size=self._batch_size)
         return self._poca_indices
 
     @poca_indices.setter
     def poca_indices(self, value: Tensor) -> None:
+        r"""Set the indices of the POCA points assigned to each voxel."""
         self._poca_indices = value
 
     @property
     def parallel_mask(self) -> Tensor:
+        r"""Tensor: The mask indicating which tracks are parallel."""
         if self._parallel_mask is None:
             self._parallel_mask = self.compute_parallel_mask(self.tracks.tracks_in, self.tracks.tracks_out)
         return self._parallel_mask
+
+    @property
+    def full_mask(self) -> Tensor:
+        r"""The full mask of POCA points."""
+        full_mask = self.compute_full_mask(mask_in_voi=self.mask_in_voi, parallel_mask=self.parallel_mask)
+
+        return full_mask
+
+    @property
+    def params(self) -> POCAParams:
+        """Return the typed params instance for this POCA variant."""
+        if self._params is None:
+            self._params = cast(P, POCAParams())
+        return self._params
+
+    @params.setter
+    def params(self, value: P) -> None:
+        """Safely update POCA parameters while preserving non-overwritten defaults."""
+        if not isinstance(value, POCAParams):
+            raise TypeError("params must be an instance of POCAParams")
+
+        if self._params is None:
+            # initialize with a default POCAParams and let the loop below overwrite fields
+            self._params = cast(P, POCAParams())
+
+        for key, val in value.__dict__.items():
+            if val is not None:
+                setattr(self._params, key, val)
+
+        self._recompute_preds = True
+
+    @property
+    def name(self) -> str:
+        r"""
+        The name of the POCA configuration based on its parameters.
+        """
+        return self.get_name_from_params()
