@@ -4,16 +4,17 @@ from typing import Tuple, Optional, Dict, Union
 import math
 import matplotlib.pyplot as plt
 import matplotlib
-import seaborn as sns
 import pandas as pd
 import numpy as np
 
 from muograph.utils.save import AbsSave
 from muograph.hits.hits import Hits
 from muograph.volume.volume import Volume
-from muograph.plotting.params import n_bins, font, alpha_sns, titlesize, hist_figsize, hist2_figsize, labelsize, tracking_figsize, configure_plot_theme
+from muograph.plotting.style import set_plot_style, subplots_4_figsize, subplots_2_figsize, n_bins, alpha
 from muograph.utils.device import DEVICE
 
+MU_MASS = 105.66  # MeV/c
+
 r"""
 Provides class for converting muon hits of the `Hits` class into
 muon tracks usable for image reconstruction purposes.
@@ -58,6 +59,7 @@ class Tracking(AbsSave):
         "measurement_type",
         "tracks_eff",
     ]
+    _maskable = ["_tracks", "_points", "_theta", "_theta_xy", "_angular_error", "_E", "tracks_eff"]
 
     def __init__(
         self,
@@ -101,6 +103,7 @@ class Tracking(AbsSave):
                     directory=self.output_dir,
                     filename=filename,
                 )
+
         elif tracks_hdf5 is not None:
             self.hits = None
             self.load_attr(attributes=self._vars_to_save, filename=tracks_hdf5)
@@ -132,6 +135,31 @@ class Tracking(AbsSave):
 
     @staticmethod
     def get_tracks_points_from_hits(hits: Tensor, chunk_size: int = 200_000) -> Tuple[Tensor, Tensor]:
+        r"""Compute muon tracks and points based on the number of the number of detection planes.
+
+        Args:
+            hits (Tensor): The hits data with shape (3, n_plane, mu).
+            chunk_size (int, optional): Size of chunks for processing in case n_mu is very large. Defaults to 200_000.
+
+        Raises:
+            ValueError: If there are less than 2 hits per event, cannot proceed to tracking.
+
+        Returns:
+            Tuple[Tensor, Tensor]: The points and tracks tensors
+            with respective size (mu, 3).
+        """
+
+        n_panels = hits.size()[1]
+
+        if n_panels > 2:
+            return Tracking.get_tracks_points_from_multi_hits(hits=hits, chunk_size=chunk_size)
+        elif n_panels == 2:
+            return Tracking.get_tracks_points_from_two_hits(hits=hits)
+        else:
+            raise ValueError("Hits have less than 2 hits per event, cannot proceed to tracking.")
+
+    @staticmethod
+    def get_tracks_points_from_multi_hits(hits: Tensor, chunk_size: int = 200_000) -> Tuple[Tensor, Tensor]:
         r"""
         The muon hits on detector planes are plugged into a linear fit
         to compute a track T(tx, ty, tz) and a point on that track P(px, py, pz).
@@ -176,6 +204,25 @@ class Tracking(AbsSave):
         tracks[:, 2] = torch.where(tracks[:, 2] > 0, -tracks[:, 2], tracks[:, 2])
         return tracks, points
 
+    @staticmethod
+    def get_tracks_points_from_two_hits(hits: Tensor) -> Tuple[Tensor, Tensor]:
+        r"""
+        The muon hits on detector planes are used
+        to compute a track T(tx, ty, tz) and a point on that track P(px, py, pz).
+
+        Args:
+            - hits (Tensor): The hits data with shape (3, n_plane, mu).
+
+        Returns:
+            - tracks, points (Tuple[Tensor, Tensor]): The points and tracks tensors
+            with respective size (mu, 3).
+        """
+
+        points = torch.mean(hits, dim=1)
+        tracks = (hits[:, -1] - hits[:, 0]) / torch.norm(hits[:, -1] - hits[:, 0], dim=0, keepdim=True)
+
+        return tracks.T, points.T
+
     @staticmethod
     def get_theta_from_tracks(tracks: Tensor) -> Tensor:
         r"""
@@ -220,8 +267,8 @@ class Tracking(AbsSave):
         r"""
         Computes the tracks efficiency.
         """
-
-        tracks_eff = torch.where(hits_eff.sum(dim=0) == 3, 1, 0)
+        n_expected_hit_per_track = hits_eff.size()[0]
+        tracks_eff = torch.where(hits_eff.sum(dim=0) == n_expected_hit_per_track, 1, 0)
         return tracks_eff
 
     @staticmethod
@@ -305,48 +352,30 @@ class Tracking(AbsSave):
         zenith_mean = zenith_angle_deg.mean()
         energy_mean = energy_gev.mean()
 
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
-
-        # Apply font globally using Matplotlib
-        import matplotlib
-
-        matplotlib.rc("font", **font)
+        set_plot_style()
 
         # Create subplots
-        fig, axs = plt.subplots(ncols=2, figsize=hist2_figsize)
-        fig.suptitle(f"Batch of {self.n_mu:,d} muons", fontsize=titlesize, fontweight="bold")
+        fig, axs = plt.subplots(ncols=2, figsize=subplots_2_figsize)
+        fig.suptitle(f"Batch of {self.n_mu:,d} muons")
 
         # Zenith angle plot
-        sns.histplot(
-            zenith_angle_deg,
-            bins=n_bins,
-            # kde=True,
-            color="blue",
-            alpha=alpha_sns,
-            ax=axs[0],
-        )
+        axs[0].hist(zenith_angle_deg, bins=n_bins, color="blue", edgecolor="black", alpha=alpha)
         axs[0].axvline(zenith_mean, color="red", linestyle="--", linewidth=1.5, label=f"Mean = {zenith_mean:.2f}°")
-        axs[0].set_xlabel(r"Zenith Angle $\theta$ [deg]", fontweight="bold")
-        axs[0].set_ylabel("Frequency [a.u]", fontweight="bold")
-        axs[0].legend(fontsize=font["size"])
+        axs[0].set_xlabel(r"Zenith Angle $\theta$ [deg]")
+        axs[0].set_ylabel("Frequency [a.u]")
+        axs[0].legend()
 
         # Energy plot
-        sns.histplot(energy_gev, bins=n_bins, color="purple", log_scale=(False, True), alpha=alpha_sns, ax=axs[1])
+        axs[1].hist(energy_gev, bins=n_bins, color="purple", edgecolor="black", log=True, alpha=alpha)
 
         axs[1].axvline(energy_mean, color="red", linestyle="--", linewidth=1.5, label=f"Mean = {energy_mean:.2f} GeV")
-        axs[1].set_xlabel("Energy [GeV]", fontweight="bold")
-        axs[1].set_ylabel("Frequency [a.u]", fontweight="bold")
-        axs[1].legend(fontsize=font["size"])
+        axs[1].set_xlabel(
+            "Energy [GeV]",
+        )
+        axs[1].set_ylabel(
+            "Frequency [a.u]",
+        )
+        axs[1].legend()
 
         # Adjust layout
         plt.tight_layout()
@@ -365,36 +394,16 @@ class Tracking(AbsSave):
         Args:
             figname (Optional[str], optional): Path to a file where to save the figure. Defaults to None.
         """
+        set_plot_style()
+
         # Extract data
         angular_error_deg = self.angular_error.detach().cpu().numpy() * 180 / math.pi
         mean = angular_error_deg.mean()
         std = angular_error_deg.std()
 
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
-
-        # Apply font globally using Matplotlib
-        import matplotlib
-
-        matplotlib.rc("font", **font)
-
         # Create the plot
-        plt.figure(figsize=hist_figsize)
-        sns.histplot(
-            angular_error_deg,
-            bins=n_bins,
-            color="blue",
-            alpha=alpha_sns,
-        )
+        plt.figure()
+        plt.hist(angular_error_deg, bins=n_bins, color="blue", alpha=alpha, edgecolor="black")
 
         # Add mean line
         plt.axvline(mean, color="red", linestyle="--", linewidth=1.5, label=f"Mean = {mean:.2f}°")
@@ -404,10 +413,14 @@ class Tracking(AbsSave):
         plt.axvline(mean + std, color="green", linestyle=":", linewidth=1.5, label=r"$\pm 1\sigma$")
 
         # Add labels, title, and legend
-        plt.title(f"Batch of {self.n_mu:,d} muons\nAngular Resolution = {std:.3f}°", fontsize=titlesize, fontweight="bold")
-        plt.xlabel(r"Angular Error $\delta\theta$ [deg]", fontweight="bold")
-        plt.ylabel("Frequency [a.u.]", fontweight="bold")
-        plt.legend(fontsize=font["size"])
+        plt.title(f"Batch of {self.n_mu:,d} muons\nAngular Resolution = {std:.3f}°")
+        plt.xlabel(
+            r"Angular Error $\delta\theta$ [deg]",
+        )
+        plt.ylabel(
+            "Frequency [a.u.]",
+        )
+        plt.legend()
 
         # Adjust layout
         plt.tight_layout()
@@ -427,21 +440,7 @@ class Tracking(AbsSave):
             figname (Optional[str], optional): Path to a file where to save the figure. Defaults to None.
         """
 
-        import matplotlib
-
-        matplotlib.rc("font", **font)
-
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
+        set_plot_style()
 
         if proj not in ("XZ", "YZ"):
             raise ValueError("proj argument must be XZ or YZ.")
@@ -451,24 +450,18 @@ class Tracking(AbsSave):
             "YZ": {"x": 1, "y": 2, "xlabel": r"$y$ [mm]", "ylabel": r"$z$ [mm]", "proj": "YZ"},
         }
 
-        fig, axs = plt.subplots(ncols=2, figsize=tracking_figsize)
-        fig.suptitle(
-            f"Tracking of event {event:,d}" + "\n" + f"{dim_map[proj]['proj']} projection, " + r"$\theta$ = " + f"{self.theta[event] * 180 / math.pi:.2f} deg",
-            fontweight="bold",
-        )
+        fig, axs = plt.subplots(ncols=3, gridspec_kw={"width_ratios": [1, 1, 0.25]})
+        axs[2].axis("off")  # Turn off the legend axis
+        fig.suptitle(f"Tracking of event {event:,d}\n{dim_map[proj]['proj']} projection, " r"$\theta$ = " + f"{self.theta[event] * 180 / math.pi:.2f} deg")
 
-        if (self.hits is None) & (hits is None):
+        if (self.hits is None) and (hits is None):
             raise ValueError("Provide hits as argument.")
-
         elif self.hits is not None:
             n_panels = self.hits.n_panels
-            # Get data as numpy array
             reco_hits_np = self.hits.reco_hits.detach().cpu().numpy()
             gen_hits_np = self.hits.gen_hits.detach().cpu().numpy()
-
         elif hits is not None:
             n_panels = hits.n_panels
-            # Get data as numpy array
             reco_hits_np = hits.reco_hits.detach().cpu().numpy()
             gen_hits_np = hits.gen_hits.detach().cpu().numpy()
 
@@ -477,44 +470,35 @@ class Tracking(AbsSave):
 
         hits_x = reco_hits_np[dim_map[proj]["x"], :, event]  # type: ignore
 
-        # Get detector span
-        x_span = abs(np.min(reco_hits_np[dim_map[proj]["x"]]) - np.max(reco_hits_np[dim_map[proj]["x"]]))  # type: ignore
+        # Calculate spans
+        x_span = abs(np.min(hits_x) - np.max(hits_x))
         y_span = abs(np.min(reco_hits_np[dim_map[proj]["y"]]) - np.max(reco_hits_np[dim_map[proj]["y"]]))  # type: ignore
 
-        # Assumes x_span > y_span
-        hits_x_span = abs(np.min(hits_x) - np.max(hits_x))
-        x_span = max(hits_x_span, y_span)
+        # Axis limits
+        x_extent = max(x_span / 1.5, 100)  # Enforce minimum extent of 100
 
-        # Set axis limits
         axs[0].set_xlim(
-            (
-                points_np[event, dim_map[proj]["x"]] - x_span / 2,  # type: ignore
-                points_np[event, dim_map[proj]["x"]] + x_span / 2,  # type: ignore
-            )
+            points_np[event, dim_map[proj]["x"]] - x_extent,  # type: ignore
+            points_np[event, dim_map[proj]["x"]] + x_extent,  # type: ignore
         )
-        for ax in axs:
-            ax.set_ylim(
-                (
-                    points_np[event, dim_map[proj]["y"]] - y_span / 1.8,  # type: ignore
-                    points_np[event, dim_map[proj]["y"]] + y_span / 1.8,  # type: ignore
-                )
-            )
-
         axs[1].set_xlim(
-            (
-                points_np[event, dim_map[proj]["x"]] - abs(np.min(hits_x) - np.max(hits_x)) / 1.5,  # type: ignore
-                points_np[event, dim_map[proj]["x"]] + abs(np.min(hits_x) - np.max(hits_x)) / 1.5,  # type: ignore
-            )
+            points_np[event, dim_map[proj]["x"]] - x_span / 1.5,  # type: ignore
+            points_np[event, dim_map[proj]["x"]] + x_span / 1.5,  # type: ignore
         )
+        for ax in axs[:2]:
+            ax.set_ylim(
+                points_np[event, dim_map[proj]["y"]] - y_span / 1.8,  # type: ignore
+                points_np[event, dim_map[proj]["y"]] + y_span / 1.8,  # type: ignore
+            )
 
-        # Plot detector panels if XZ or YZ projection
-        for ax in axs:
+        # Detector panels
+        for ax in axs[:2]:
             for i in range(n_panels):
                 label = "Detector panel" if i == 0 else None
                 ax.axhline(y=np.mean(reco_hits_np[dim_map[proj]["y"], i]), label=label, alpha=0.4)  # type: ignore
 
-        for ax in axs:
-            # Plot reco hits
+        # Plot hits, points, and tracks
+        for ax in axs[:2]:
             ax.scatter(
                 x=reco_hits_np[dim_map[proj]["x"], :, event],  # type: ignore
                 y=reco_hits_np[dim_map[proj]["y"], :, event],  # type: ignore
@@ -524,8 +508,6 @@ class Tracking(AbsSave):
                 alpha=0.5,
                 s=80,
             )
-
-            # Plot gen hits
             ax.scatter(
                 x=gen_hits_np[dim_map[proj]["x"], :, event],  # type: ignore
                 y=gen_hits_np[dim_map[proj]["y"], :, event],  # type: ignore
@@ -534,8 +516,6 @@ class Tracking(AbsSave):
                 marker="+",
                 alpha=0.5,
             )
-
-            # Plot fitted point
             ax.scatter(
                 x=points_np[event, dim_map[proj]["x"]],  # type: ignore
                 y=points_np[event, dim_map[proj]["y"]],  # type: ignore
@@ -544,10 +524,6 @@ class Tracking(AbsSave):
                 marker="x",
                 s=100,
             )
-
-            ax.set_xlabel(dim_map[proj]["xlabel"], fontweight="bold")
-            ax.set_ylabel(dim_map[proj]["ylabel"], fontweight="bold")
-
             ax.plot(
                 [
                     points_np[event, dim_map[proj]["x"]] - tracks_np[event, dim_map[proj]["x"]] * 1000,  # type: ignore
@@ -562,11 +538,23 @@ class Tracking(AbsSave):
                 linestyle="--",
                 label="Fitted track",
             )
+            ax.set_xlabel(
+                dim_map[proj]["xlabel"],
+            )
+            ax.set_ylabel(
+                dim_map[proj]["ylabel"],
+            )
 
         axs[0].set_aspect("equal")
 
+        # Collect unique labels and handles
+        handles, labels = axs[0].get_legend_handles_labels()
+        unique = dict(zip(labels, handles))  # Remove duplicates while preserving order
+
+        # Show the legend on the third axis
+        axs[2].legend(unique.values(), unique.keys(), loc="center left", frameon=False)
+
         plt.tight_layout()
-        axs[0].legend(loc="center right", bbox_to_anchor=(0.25, 1.2))
 
         if figname is not None:
             plt.savefig(figname, bbox_inches="tight")
@@ -589,14 +577,15 @@ class Tracking(AbsSave):
 
         # Set attributes without setter method to None
         self._reset_vars()
+        n_muons = self.tracks.size(0)
 
-        n_muons = self.tracks.size()[0]
-        # Loop over class attributes and apply the mask is Tensor
-        for var in vars(self).keys():
-            data = getattr(self, var)
-            if isinstance(data, Tensor):
-                if data.size()[0] == n_muons:
-                    setattr(self, var, data[mask])
+        for var in self._maskable:
+            data = getattr(self, var, None)
+            if isinstance(data, Tensor) and data.size(0) == n_muons:
+                setattr(self, var, data[mask])
+
+        if self.hits is not None:
+            self.hits._filter_events(mask)
 
     @property
     def df(self) -> pd.DataFrame:
@@ -698,7 +687,7 @@ class Tracking(AbsSave):
         The angular error between the generated and reconstructed tracks.
         """
         if self._angular_error is None:
-            if self.hits.spatial_res.sum() == 0.0:
+            if self.hits is None or self.hits.spatial_res.sum() == 0.0:
                 self._angular_error = torch.zeros_like(self.theta)
             else:
                 self._angular_error = self.get_angular_error(self.theta)
@@ -753,10 +742,7 @@ class TrackingMST:
 
     _vars_to_load = ["tracks", "points", "angular_res", "E", "tracks_eff", "hits"]
 
-    def __init__(
-        self,
-        trackings: Tuple[Tracking, Tracking] = None,
-    ) -> None:
+    def __init__(self, trackings: Tuple[Tracking, Tracking]) -> None:
         r"""
         Initializes the TrackingMST object with 2 instances of the Tracking class
         (with tags 'above' and 'below').
@@ -766,13 +752,9 @@ class TrackingMST:
             for the incoming muon tracks (Tracking.label = 'above') and outgoing tracks
             (Tracking.label = 'below')
         """
-
         # Load data from Tracking instances
-        if trackings is not None:
-            for tracking, tag in zip(trackings, ["_in", "_out"]):
-                self.load_attr_from_tracking(tracking, tag)
-        else:
-            raise ValueError("Provide instance of Tracking class with label `above` and `below`.")
+        for tracking, tag in zip(trackings, ["_in", "_out"]):
+            self.load_attr_from_tracking(tracking, tag)
 
         # Filter muon event due to detector efficiency
         self.n_mu_removed = (self.n_mu - self.muon_eff.sum()).detach().cpu().item()
@@ -887,78 +869,60 @@ class TrackingMST:
     ) -> None:
         r"""
         Plot the zenith angle and energy of the reconstructed tracks.
+
         Args:
             figname (str): If provided, save the figure at figname.
         """
 
-        matplotlib.rc("font", **font)
+        set_plot_style()
 
-        sns.set_theme(
-            style="darkgrid",
-            rc={
-                "font.family": font["family"],
-                "font.size": font["size"],
-                "axes.labelsize": font["size"],  # Axis label font size
-                "axes.titlesize": font["size"],  # Axis title font size
-                "xtick.labelsize": font["size"],  # X-axis tick font size
-                "ytick.labelsize": font["size"],  # Y-axis tick font size
-            },
-        )
-
-        fig, axs = plt.subplots(ncols=2, nrows=2, figsize=(2 * hist_figsize[0], 2 * hist_figsize[1]))
+        fig, axs = plt.subplots(ncols=2, nrows=2, figsize=subplots_4_figsize)
         axs = axs.ravel()
 
-        # Fig title
-        fig.suptitle(f"Batch of {self.n_mu:,d} muons", fontsize=titlesize, fontweight="bold")
+        fig.suptitle(f"Batch of {self.n_mu:,d} muons")
 
         # Zenith angle
-        sns.histplot(data=self.theta_in.detach().cpu().detach().numpy() * 180 / math.pi, alpha=alpha_sns, bins=n_bins, ax=axs[0], color="blue")
-
+        data_theta = self.theta_in.detach().cpu().numpy() * 180 / math.pi
+        axs[0].hist(data_theta, bins=n_bins, color="blue", edgecolor="black", alpha=alpha)
         axs[0].axvline(
-            x=self.theta_in.mean().detach().cpu().numpy() * 180 / math.pi,
-            label=f"mean = {self.theta_in.mean().detach().cpu().numpy() * 180 / math.pi:.1f} deg",
+            x=data_theta.mean(),
+            label=f"mean = {data_theta.mean():.1f} deg",
             color="red",
         )
-        axs[0].set_xlabel(r" Zenith angle $\theta$ [deg]", fontweight="bold")
+        axs[0].set_xlabel(
+            r" Zenith angle $\theta$ [deg]",
+        )
 
         # Energy
-        sns.histplot(
-            data=self.E.detach().cpu().detach().numpy() / 1000,
-            alpha=alpha_sns,
-            bins=n_bins,
-            ax=axs[1],
-            log_scale=(False, True),
-            color="purple",
-        )
+        data_E = self.E.detach().cpu().numpy() / 1000
+        axs[1].hist(data_E, bins=n_bins, log=True, color="purple", edgecolor="black", alpha=alpha)
         axs[1].axvline(
-            x=self.E.mean().detach().cpu().numpy() / 1000,
-            label=f"mean = {self.E.mean().detach().cpu().numpy() / 1000:.2f} GeV",
+            x=data_E.mean(),
+            label=f"mean = {data_E.mean():.2f} GeV",
             color="red",
         )
-        axs[1].set_xlabel(r" Energy [GeV]", fontweight="bold")
+        axs[1].set_xlabel(
+            r" Energy [GeV]",
+        )
 
         # Scattering angle
-        sns.histplot(
-            data=self.dtheta.detach().cpu().detach().numpy() * 180 / math.pi,
-            alpha=alpha_sns,
-            bins=n_bins,
-            ax=axs[2],
-            log_scale=(False, True),
-            color="green",
-        )
+        data_dtheta = self.dtheta.detach().cpu().numpy() * 180 / math.pi
+        axs[2].hist(data_dtheta, bins=n_bins, log=True, color="green", edgecolor="black", alpha=alpha)
         axs[2].axvline(
-            x=self.dtheta.mean().detach().cpu().numpy() * 180 / math.pi,
-            label=f"mean = {self.dtheta.mean().detach().cpu().numpy() * 180 / math.pi:.3f} deg",
+            x=data_dtheta.mean(),
+            label=f"mean = {data_dtheta.mean():.3f} deg",
             color="red",
         )
-        axs[2].set_xlabel(r" Scattering angle $\delta\theta$ [deg]", fontweight="bold", fontsize=font["size"])
+        axs[2].set_xlabel(r" Scattering angle $\delta\theta$ [deg]")
 
+        # Style formatting
         for ax in axs[:-1]:
             ax.grid(visible=True, color="grey", linestyle="--", linewidth=0.5)
-            ax.set_ylabel("Frequency [a.u]", fontweight="bold", fontsize=font["size"])
-            ax.tick_params(axis="both", labelsize=labelsize)
+            ax.set_ylabel("Frequency [a.u]")
+            ax.tick_params(axis="both")
             ax.legend()
 
+        # Remove unused subplot
         axs[-1].remove()
         axs[-1] = None
 
@@ -1040,7 +1004,7 @@ class TrackingMST:
             voi (Optional[Volume], optional): An instance of the Volume class. If provided, gets represented on the plot. Defaults to None.
             figname (Optional[str], optional): If provided, the figure is saved as `figname`. Defaults to None.
         """
-        configure_plot_theme(font=font)  # type: ignore
+        set_plot_style()
 
         # Numpy data
         points_in_np = self.points_in.detach().cpu().numpy()
@@ -1062,14 +1026,13 @@ class TrackingMST:
 
         dim_xy = (int(dim_map[proj]["x"]), int(dim_map[proj]["y"]))
 
-        fig, ax = plt.subplots(figsize=tracking_figsize)
+        fig, ax = plt.subplots()
         fig.suptitle(
             f"Tracking of event {event:,d}"
             + "\n"
             + f"{dim_map[proj]['proj']} projection, "
             + r"$\delta\theta$ = "
             + f"{self.dtheta[event] * 180 / math.pi:.2f} deg",
-            fontweight="bold",
         )
 
         # Get plot xy span
@@ -1110,8 +1073,8 @@ class TrackingMST:
             bbox_to_anchor=(1.0, 0.7),
         )
         ax.set_aspect("equal")
-        ax.set_xlabel(f"{dim_map[proj]['xlabel']}", fontweight="bold")
-        ax.set_ylabel(f"{dim_map[proj]['ylabel']}", fontweight="bold")
+        ax.set_xlabel(f"{dim_map[proj]['xlabel']}")
+        ax.set_ylabel(f"{dim_map[proj]['ylabel']}")
 
         if figname is not None:
             plt.savefig(figname, bbox_inches="tight")
@@ -1150,7 +1113,7 @@ class TrackingMST:
     @property
     def E(self) -> Tensor:
         r"""
-        Muons kinetic energy.
+        Muons kinetic energy in MeV/c2.
         """
         return self._E
 
@@ -1158,6 +1121,11 @@ class TrackingMST:
     def E(self, value: Tensor) -> None:
         self._E = value
 
+    @property
+    def p(self) -> Tensor:
+        r"""Muon momentum in MeV/c."""
+        return MU_MASS * torch.sqrt(((self.E / MU_MASS) + 1) ** 2 - 1)
+
     # Scattering angle
     @property
     def dtheta(self) -> Tensor:
@@ -1166,6 +1134,10 @@ class TrackingMST:
             self._dtheta = self.compute_dtheta_from_tracks(self.tracks_in, self.tracks_out)
         return self._dtheta
 
+    @dtheta.setter
+    def dtheta(self, value: Tensor) -> None:
+        self._dtheta = value
+
     # Tracks
     @property
     def tracks_in(self) -> Tensor:
